/*
 * table.c --
 *
 *      Datatype to store contents of a mib table- 
 *
 * Copyright (c) 2002 T. Klie, Technical University of Braunschweig.
 *
 * See the file "COPYING" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * @(#) $Id: table.c 1716 2002-12-16 17:08:30Z tklie $
 */

#include <malloc.h>
#include "table.h"

struct MDTable
*md_createTable( char *name, oid *prefix, unsigned int prefix_len )
{
    struct MDTable *ret = (struct MDTable *)malloc( sizeof( struct MDTable ) );
    if( ret ) {
	ret->name = name;
	ret->prefix = prefix;
	ret->prefix_len = prefix_len;
	ret->rows = NULL;
	ret->maxColNum = 0;
	ret->augmentations = NULL;
    }
    return ret;
}

static MDRow*
createRow( oid *index, size_t index_len )
{
    MDRow *newRow;
    
    /* create new row */
    newRow = (struct MDRow *)malloc( sizeof( struct MDRow ) );
    if( !newRow ) {
	fprintf( stderr, "Warning! Malloc failed in table.c\n" );
	return NULL;
    }

    /* fill in values */
    newRow->index = index;
    newRow->index_len = index_len;
    newRow->columns = NULL;
    newRow->next = NULL;

    return newRow;
}


MDRow
*md_addRow( MDTable *table, oid *index, unsigned int index_len )
{
    MDRow *iterRow = NULL, *oldIterRow = NULL;

    /* find last row in list */
    for( iterRow = table->rows; iterRow; iterRow = iterRow->next ) {
	if(! memcmp( iterRow->index, index,
		      MIN( index_len, iterRow->index_len ) * sizeof( oid ) ) ){
	    /* row already exists */
	    return iterRow; /* xxx should we give a warning? */
	}
	oldIterRow = iterRow;
    }

    /* create new row */
    iterRow = createRow( index, index_len );

    /* connect row to list */
    if( oldIterRow ) {
	/* table already has rows */
	oldIterRow->next = iterRow;
    }
    else {
	/* table has no rows yet */
	table->rows = iterRow;
    }

    return iterRow;
}




MDRow
*md_findRow( MDTable *table, oid *index, unsigned int index_len )
{
    MDRow *iterRow = NULL;

    if( ! table ) {
	return NULL;
    }

    for( iterRow = table->rows; iterRow; iterRow = iterRow->next ) {
	if( iterRow->index_len == index_len ) {
	    if( ! memcmp( iterRow->index, index,
			  index_len * sizeof( oid ) ) ) {
		return iterRow;
	    }
	}
    }
    
    return NULL;
}


MDCol
*md_addColumn( MDRow *row, oid col, char *value )
{
    MDCol *iterCol, *oldIterCol = NULL, *newCol;

    if( ! row ) {
	return NULL;
    }

    /* find las column in list */
    for( iterCol = row->columns; iterCol; iterCol = iterCol->next ) {
	if( col == iterCol->col ) {
	    /* column already exists */
	    return iterCol; /* xxx a warning? */
	}
	oldIterCol = iterCol;
    }
    
    /* create new column */
    newCol = (struct MDCol *)malloc( sizeof( struct MDCol ) );
    if( ! newCol ) {
	fprintf( stderr, "Warning! Malloc failed in table.c!\n" );
	return NULL;
    }
    
    /* fill in values */
    newCol->col = col;
    newCol->value = value;
    newCol->next = NULL;
    newCol->subRow = NULL;
    newCol->smiNode = NULL;

    /* connect column to list */
    if( oldIterCol ) {
	/* row already has columns */
	oldIterCol->next = newCol;
    }
    else {
	/* row has no columns yet */
	row->columns = newCol;
    }

    return newCol;
}

MDCol
*md_findColumn( MDRow *row, oid col )
{
    MDCol *iterCol;

    if( ! row ) {
	return NULL;
    }
    
    for( iterCol = row->columns; iterCol; iterCol = iterCol->next ) {
	if( iterCol->col == col ) {
	    return iterCol;
	}
    }
    return NULL;
}

MDAugment
*md_getAugment( MDTable *table, MDCol *col )
{
    MDAugment *iterAugm, *oldAugm;
       
    if( !table->augmentations || col->col < table->augmentations->startCol ) {
	return NULL;
    }

    oldAugm = table->augmentations;
    
    for( iterAugm = oldAugm->next; iterAugm; iterAugm =  iterAugm->next ) {
	if( col->col < iterAugm->startCol ) {
	    return oldAugm;
	}
	oldAugm = iterAugm;
    }
    return NULL;
}

void
md_dropTable( MDTable *table )
{
    MDRow *iterRow, *oldIterRow = NULL;
    MDCol *iterCol, *oldIterCol = NULL;

    for( iterRow = table->rows; iterRow; iterRow = iterRow->next ) {
	for( iterCol = iterRow->columns; iterCol; iterCol = iterCol->next ) {
	    if( oldIterCol ) {
		free( oldIterCol );
	    }
	    oldIterCol = iterCol;
	}
	if( oldIterRow ) {
	    free( oldIterRow );
	}
	oldIterRow = iterRow;
    }
    
    free( table );
}


unsigned int
md_numColumns( MDRow *row )
{
    MDCol *iterCol;
    unsigned int ret = 0;

    for( iterCol = row->columns; iterCol; iterCol = iterCol->next ) {
	ret++;
    }
    return ret;
    
}
